@@ -1,16 +1,15 @@
 """This module encapsulate the basic elements of the UCCA annotation.
 
 A UCCA annotation is practically a directed acyclic graph (DAG), which
-represents a :class:Passage of text and its annotation. The annotation itself
-is divided into :class:Layer objects, where in each layer :class:Node objects
+represents a :class:`Passage` of text and its annotation. The annotation itself
+is divided into :class:`Layer` objects, where in each layer :class:`Node` objects
 are connected between themselves and to Nodes in other layers using
-:class:Edge objects.
+:class:`Edge` objects.
 
 """
 
 import functools
 
-
 # Max number of digits allowed for a unique ID
 UNIQUE_ID_MAX_DIGITS = 5
 
@@ -19,12 +18,12 @@ IRRELEVANT_ATTRIBUTES = {"uncertain"}
 
 
 # Used as the default ordering key function for ordered objects, namely
-# :class:Layer and :class:Node .
+# :class:`Layer` and :class:`Node` .
 def id_orderkey(node):
     """Key function which sorts by layer (string), then by unique ID (int).
 
     Args:
-        node: :class:Node which we will to sort according to its ID
+        node: :class:`Node` which we will to sort according to its ID
 
     Returns:
         a string with the layer and unique ID in such a way that sort will
@@ -36,11 +35,11 @@ def id_orderkey(node):
 
 
 def edge_id_orderkey(edge):
-    """Key function which sorts Edges by its IDs (using :func:id_orderkey).
+    """Key function which sorts Edges by its IDs (using :func:`id_orderkey`).
 
     Args:
-        edge: :class:Edge which we wish to sort according to the ID of its
-        parent and children after using :func:id_orderkey.
+        edge: :class:`Edge` which we wish to sort according to the ID of its
+        parent and children after using :func:`id_orderkey`.
 
     Returns:
         a string with the layer and unique ID in such a way that sort will
@@ -57,7 +56,7 @@ class UCCAError(Exception):
 
 
 class FrozenPassageError(UCCAError):
-    """Exception raised when trying to modify a frozen :class:Passage."""
+    """Exception raised when trying to modify a frozen :class:`Passage`."""
     pass
 
 
@@ -65,7 +64,7 @@ class DuplicateIdError(UCCAError):
     """Exception raised when trying to add an element with an existing ID.
 
     For each element, a unique ID must be assigned. If the ID of the new element
-    is already present in the :class:Passage in some way, this exception is
+    is already present in the :class:`Passage` in some way, this exception is
     raised.
 
     """
@@ -73,7 +72,7 @@ class DuplicateIdError(UCCAError):
 
 
 class MissingNodeError(UCCAError):
-    """Exception raised when trying to access a non-existent :class:Node."""
+    """Exception raised when trying to access a non-existent :class:`Node`."""
     pass
 
 
@@ -83,10 +82,10 @@ class UnimplementedMethodError(UCCAError):
 
 
 class ModifyPassage:
-    """Decorator for changing a :class:Passage or any member of it.
+    """Decorator for changing a :class:`Passage` or any member of it.
 
     This decorator is mandatory for anything which causes the elements in
-    a :class:Passage to change by adding or removing an element, or changing
+    a :class:`Passage` to change by adding or removing an element, or changing
     an attribute.
 
     It validates that the Passage is not frozen before allowing the change.
@@ -110,24 +109,23 @@ class ModifyPassage:
         return functools.partial(self.__call__, obj)
 
     def __call__(self, *args, **kwargs):
-        """Decorating functions which modify :class:Passage elements.
+        """Decorating functions which modify :class:`Passage` elements.
 
         :param args: list of all arguments, assuming the first is the object
-                which modifies :class:Passage, and it has an attribute root
+                which modifies :class:`Passage`, and it has an attribute root
                 which points to the Passage it is part of.
         :param kwargs: list of all keyword arguments
-
-        :return The decorated function result.
-
-        :raise FrozenPassageError: if the :class:Passage is frozen and can't be
+        :return: The decorated function result.
+        :raise FrozenPassageError: if the :class:`Passage` is frozen and can't be
                 modified.
-
         """
+
         @functools.wraps(self.fn)
         def decorated(*args, **kwargs):
             if args[0].root.frozen:
-                raise FrozenPassageError()
+                raise FrozenPassageError(args[0].root.ID)
             return self.fn(*args, **kwargs)
+
         return decorated(*args, **kwargs)
 
 
@@ -136,7 +134,7 @@ class _AttributeDict:
 
     This dictionary is used to store attributes which are part of any
     element in the UCCA annotation scheme. It's advantage over regular
-    dictionary is adhering to :class:Passage frozen status and modification
+    dictionary is adhering to :class:`Passage` frozen status and modification
     decorators.
 
     Attributes:
@@ -158,12 +156,12 @@ class _AttributeDict:
         """True iff the two objects are equal (only dicts, w.o.r.t Passage).
 
         :param other: AttributeDict to compare to
-
-        :return True iff the dictionaries are equal.
-
+        :return: True iff the dictionaries are equal.
         """
+
         def omit_irrelevant(d):
             return {k: v for k, v in d.items() if k not in IRRELEVANT_ATTRIBUTES}
+
         return omit_irrelevant(self._dict) == omit_irrelevant(other._dict)
 
     @property
@@ -192,13 +190,58 @@ class _AttributeDict:
         return self._dict.items()
 
 
-class Edge:
-    """Labeled edge between two :class:Node objects in UCCA annotation graph.
+class Category:
+    """when considering refinement layers, each edge can have multiple tags sorted in a certain hierarchy.
+    for this reason, a category must include not only the tag information but also the layer and hierarchy
+    information.
+    """
 
-    An edge between Nodes in a :class:Passage is a simple object; it is a
+    def __init__(self, tag, slot=None, layer=None, parent=None):
+        self._tag = tag
+        self._slot = slot if slot else ""
+        self._layer = layer if layer else ""
+        self._parent = parent if parent else ""
+        self.extra = {}
+
+    @property
+    def tag(self):
+        return self._tag
+
+    @tag.setter
+    def tag(self, new_tag):
+        self._tag = new_tag
+
+    @property
+    def slot(self):
+        return self._slot
+
+    @property
+    def layer(self):
+        return self._layer
+
+    @property
+    def parent(self):
+        return self._parent
+
+    @parent.setter
+    def parent(self, new_parent):
+        self._parent = new_parent
+
+    def to_xml(self):
+        pass
+
+    def __iter__(self):
+        return iter((self.tag, self.slot, self.layer, self.parent))
+
+
+class Edge:
+    """Labeled edge between two :class:`Node` objects in UCCA annotation graph.
+
+    An edge between Nodes in a :class:`Passage` is a simple object; it is a
     directed edge whose ID is derived by the parent and child of the edge,
     it is mostly immutable except for its attributes, and it is labeled with
-    the connection type between the Nodes.
+    the connection type between the Nodes. An edge can have multiple annotations, representing
+    connection types of one or more layers.
 
     Attributes:
         ID: ID of the Edge, constructed from the IDs of the two Nodes
@@ -208,43 +251,48 @@ class Edge:
         tag: the string label of the Edge
         parent: the originating Node of the Edge
         child: the target Node of the Edge
+        categories: a list of categories for this edge
         ID_FORMAT: format string which creates the ID of the Edge from
-            the IDs of the parent (first argument to the formattinf string)
+            the IDs of the parent (first argument to the formatting string)
             and the child (second argument).
 
     """
 
     ID_FORMAT = "{}->{}"
 
-    def __init__(self, root, tag, parent, child, attrib=None):
-        """Creates a new :class:Edge object.
+    def __init__(self, root, parent, child, tag=None, attrib=None):
+        """Creates a new :class:`Edge` object.
 
-        :param see :class:Edge documentation.
+        :param see :class:`Edge` documentation.
 
-        :raise FrozenPassageError: if the :class:Passage object we are part of
+        :raise FrozenPassageError: if the :class:`Passage` object we are part of
                 is frozen and can't be modified.
 
         """
         if root.frozen:
-            raise FrozenPassageError()
-        self._tag = tag
+            raise FrozenPassageError(root.ID)
         self._root = root
         self._parent = parent
         self._child = child
         self._attrib = _AttributeDict(root, attrib)
+        self._categories = [Category(tag)] if tag else []
         self.extra = {}
 
     @property
     def tag(self):
-        return self._tag
+        return self.categories[0].tag
 
     @tag.setter
     @ModifyPassage
     def tag(self, new_tag):
-        old_tag = self._tag
-        self._tag = new_tag
+        old_tag = self.tag
+        self.categories[0].tag = new_tag
         self._root._change_edge_tag(self, old_tag)
 
+    @property
+    def tags(self):
+        return [category.tag for category in self.categories]
+
     @property
     def root(self):
         return self._root
@@ -253,6 +301,18 @@ class Edge:
     def parent(self):
         return self._parent
 
+    @property
+    def categories(self):
+        try:
+            return self._categories
+        except AttributeError as e:
+            raise IOError("Load passage %s from pickle saved with UCCA<1.1. Please load from XML instead." %
+                          self.root.ID) from e
+
+    @categories.setter
+    def categories(self, new_categories):
+        self._categories = new_categories
+
     @property
     def child(self):
         return self._child
@@ -281,7 +341,7 @@ class Edge:
         :param ignore_edge: function that returns whether to ignore a given edge
 
 
-        :return True iff the Edges are equal.
+        :return: True iff the Edges are equal.
 
         """
         return self.tag == other.tag and \
@@ -291,17 +351,31 @@ class Edge:
                                   ordered=ordered,
                                   ignore_node=ignore_node, ignore_edge=ignore_edge))
 
+    @ModifyPassage
+    def add(self, tag, slot="", layer="", parent=""):
+        """ adds a new category to the edge"""
+        c = Category(tag, slot, layer, parent)
+        self.categories.append(c)
+        if c.tag not in self.root.categories:
+            self.root._update_categories(c)
+        if c.parent and c.parent not in self.root.refined_categories:
+            self.root._update_refined_categories(c.parent)
+        return c
+
     def __repr__(self):
         return self.ID
 
+    def __getitem__(self, index):
+        return self.categories[index]
+
 
 class Node:
     """Labeled Node in UCCA annotation graph.
 
-    A Node in :class:Passage UCCA annotation is an vertex in the annotation
+    A Node in :class:`Passage` UCCA annotation is an vertex in the annotation
     graph, which may be an internal vertex or a leaf, and is labeled with a
-    tag that specifies both the :class:Layer it belongs to and it's ID in this
-    Layer. It can have multiple children Nodes through :class:Edge objects,
+    tag that specifies both the :class:`Layer` it belongs to and it's ID in this
+    Layer. It can have multiple children Nodes through :class:`Edge` objects,
     and these children are ordered according to an internal order function.
 
     Attributes:
@@ -326,16 +400,16 @@ class Node:
 
     def __init__(self, ID, root, tag, attrib=None, *,
                  orderkey=edge_id_orderkey):
-        """Creates a new :class:Node object.
+        """Creates a new :class:`Node` object.
 
-        :param see :class:Node documentation.
+        :param see :class:`Node` documentation.
 
-        :raise FrozenPassageError: if the :class:Passage object we are part of
+        :raise FrozenPassageError: if the :class:`Passage` object we are part of
                 is frozen and can't be modified.
 
         """
         if root.frozen:
-            raise FrozenPassageError()
+            raise FrozenPassageError(root.ID)
         self._tag = tag
         self._root = root
         self._ID = ID
@@ -347,7 +421,10 @@ class Node:
 
         # After properly initializing self, add it to the Passage/Layer
         root._add_node(self)
-        root.layer(self.layer.ID)._add_node(self)
+        try:
+            root.layer(self.layer.ID)._add_node(self)
+        except KeyError as e:
+            raise ValueError("Invalid layer '%s' in node ID '%s'" % (self.layer.ID, self._ID)) from e
 
     @property
     def tag(self):
@@ -405,23 +482,25 @@ class Node:
         return Node.__name__ + "(" + self.ID + ")"
 
     @ModifyPassage
-    def add(self, edge_tag, node, *, edge_attrib=None):
-        """Adds another :class:Node object as a child of self.
+    def add_multiple(self, edge_categories, node, *, edge_attrib=None):
+        """Adds another :class:`Node` object as a child of self.
 
-        :param edge_tag: the label of the :class:Edge connecting between the
-                Nodes
-            node: the Node object which we want to have an Edge to
-            edge_attrib: Keyword only, dictionary of attributes to be passed
+        :param edge_categories: a list of 4-tuples representing the categories on this edge of the :class:`Edge`
+                connecting between the Nodes
+        :param node: the Node object which we want to have an Edge to
+        :param edge_attrib: Keyword only, dictionary of attributes to be passed
                 to the Edge initializer.
 
-        :return the newly created Edge object
+        :return: the newly created Edge object
 
-        :raise FrozenPassageError: if the :class:Passage object we are part of
+        :raise FrozenPassageError: if the :class:`Passage` object we are part of
                 is frozen and can't be modified.
 
         """
-        edge = Edge(root=self._root, tag=edge_tag, parent=self,
+        edge = Edge(root=self._root, parent=self,
                     child=node, attrib=edge_attrib)
+        for category in edge_categories:
+            edge.add(*category)
         self._outgoing.append(edge)
         self._outgoing.sort(key=self._orderkey)
         node._incoming.append(edge)
@@ -429,12 +508,30 @@ class Node:
         self.root._add_edge(edge)
         return edge
 
+    @ModifyPassage
+    def add(self, tag, node, *, edge_attrib=None):
+        """Adds another :class:`Node` object as a child of self.
+
+        :param edge_categories: a list of 4-tuples representing the categories on this edge of the :class:`Edge`
+                connecting between the Nodes
+            node: the Node object which we want to have an Edge to
+            edge_attrib: Keyword only, dictionary of attributes to be passed
+                to the Edge initializer.
+
+        :return: the newly created Edge object
+
+        :raise FrozenPassageError: if the :class:`Passage` object we are part of
+                is frozen and can't be modified.
+
+        """
+        return self.add_multiple([(tag,)], node, edge_attrib=edge_attrib)
+
     @ModifyPassage
     def remove(self, edge_or_node):
-        """Removes the :class:Edge between self and a child :class:Node.
+        """Removes the :class:`Edge` between self and a child :class:`Node`.
 
         This methods removes the Edge given, or the Edge connecting self and
-        the Node given, from the annotation of :class:Passage. It does not
+        the Node given, from the annotation of :class:`Passage`. It does not
         remove the target or originating Node from the graph but just unlinks
         them.
 
@@ -447,8 +544,8 @@ class Node:
             try:
                 edge = [edge for edge in self._outgoing
                         if edge.child == edge_or_node][0]
-            except IndexError:
-                raise MissingNodeError()
+            except IndexError as e:
+                raise MissingNodeError(edge_or_node) from e
         else:  # an Edge object
             edge = edge_or_node
 
@@ -456,8 +553,8 @@ class Node:
             self._outgoing.remove(edge)
             edge.child._incoming.remove(edge)
             self.root._remove_edge(edge)
-        except ValueError:
-            raise MissingNodeError()
+        except ValueError as e:
+            raise MissingNodeError(edge_or_node) from e
 
     @property
     def orderkey(self):
@@ -470,10 +567,10 @@ class Node:
 
     @ModifyPassage
     def destroy(self):
-        """Removes the :class:Node from the :class:Passage annotation graph.
+        """Removes the :class:`Node` from the :class:`Passage` annotation graph.
 
-        This method unlinks self from all other :class:Node objects and removes
-        self from the :class:Layer and Passage objects.
+        This method unlinks self from all other :class:`Node` objects and removes
+        self from the :class:`Layer` and Passage objects.
 
         """
         # using outgoing and incoming so I won't change the list I'm working on
@@ -502,7 +599,7 @@ class Node:
         :param ignore_node: function that returns whether to ignore a given node
         :param ignore_edge: function that returns whether to ignore a given edge
 
-        :return True iff the Nodes are equal in the terms given.
+        :return: True iff the Nodes are equal in the terms given.
 
         """
         if self.tag != other.tag or not self._attrib.equals(other._attrib):
@@ -512,8 +609,8 @@ class Node:
         edges, other_edges = [[edge for edge in node
                                if (ignore_node is None or
                                    not ignore_node(edge.child)) and (
-                                   ignore_edge is None or
-                                   not ignore_edge(edge))]
+                                       ignore_edge is None or
+                                       not ignore_edge(edge))]
                               for node in (self, other)]
         if len(edges) != len(other_edges):
             return False  # not necessary, but gives better performance
@@ -541,7 +638,7 @@ class Node:
         :param other: the Node object to compare to
         :param ignore_node: function that returns whether to ignore a given node
 
-        :return List of edges present in this node but missing in the other.
+        :return: List of edges present in this node but missing in the other.
 
         """
         edges, other_edges = [[edge for edge in node
@@ -552,7 +649,7 @@ class Node:
                       key=edge_id_orderkey)
 
     def iter(self, obj="nodes", method="dfs", duplicates=False, key=None):
-        """Iterates the :class:Node objects in the subtree of self.
+        """Iterates the :class:`Node` objects in the subtree of self.
 
         :param obj: yield Node objects (use value "nodes", default) or Edge
                 objects (use values "edges")
@@ -568,7 +665,7 @@ class Node:
                 is still iterated.  Defaults to None (returns all items).
 
         Yields:
-            a :class:Node or :class:Edge object according to the iteration
+            a :class:`Node` or :class:`Edge` object according to the iteration
             parameters.
 
         """
@@ -593,12 +690,16 @@ class Node:
             else:
                 waiting = to_add + waiting
 
+    def get_terminals(self, *args, **kwargs):
+        """Returns a list of all terminals under the span of this Node."""
+        return [t for e in self._outgoing for t in e.child.get_terminals(*args, **kwargs)]
+
 
 class Layer:
-    """Group of similar :class:Node objects in UCCA annotation graph.
+    """Group of similar :class:`Node` objects in UCCA annotation graph.
 
-    A Layer in UCCA annotation graph is a subgraph of the whole :class:Passage
-    annotation graph which consists of similar Nodes and :class:Edge objects
+    A Layer in UCCA annotation graph is a subgraph of the whole :class:`Passage`
+    annotation graph which consists of similar Nodes and :class:`Edge` objects
     between them. The Nodes and the Layer itself has some formal definition for
     being grouped together.
 
@@ -620,16 +721,16 @@ class Layer:
     """
 
     def __init__(self, ID, root, attrib=None, *, orderkey=id_orderkey):
-        """Creates a new :class:Layer object.
+        """Creates a new :class:`Layer` object.
 
-        :param see :class:Layer documentation.
+        :param see :class:`Layer` documentation.
 
-        :raise FrozenPassageError: if the :class:Passage object we are part of
+        :raise FrozenPassageError: if the :class:`Passage` object we are part of
                 is frozen and can't be modified.
 
         """
         if root.frozen:
-            raise FrozenPassageError()
+            raise FrozenPassageError(root.ID)
         self._ID = ID
         self._root = root
         self._attrib = _AttributeDict(root, attrib)
@@ -683,7 +784,7 @@ class Layer:
         :param ignore_node: function that returns whether to ignore a given node
         :param ignore_edge: function that returns whether to ignore a given edge
 
-        :return True iff self and other are Layer-equal.
+        :return: True iff self and other are Layer-equal.
 
         """
         if not self._attrib.equals(other._attrib):
@@ -710,9 +811,9 @@ class Layer:
         return not other_heads
 
     def _add_edge(self, edge):
-        """Alters self.heads if an :class:Edge has been added to the subgraph.
+        """Alters self.heads if an :class:`Edge` has been added to the subgraph.
 
-        Should be called when both :class:Node objects of the edge are part
+        Should be called when both :class:`Node` objects of the edge are part
         of this Layer (and hence part of the subgraph of it).
 
         :param edge: the Edge added to the Layer subgraph
@@ -725,9 +826,9 @@ class Layer:
         self._heads.sort(key=self._orderkey)
 
     def _remove_edge(self, edge):
-        """Alters self.heads if an :class:Edge has been removed.
+        """Alters self.heads if an :class:`Edge` has been removed.
 
-        Should be called when the child :class:Node object of the edge is part
+        Should be called when the child :class:`Node` object of the edge is part
         of this Layer (and hence part of the subgraph of it).
 
         :param edge: the Edge removed from the Layer subgraph
@@ -741,9 +842,9 @@ class Layer:
         self._heads.sort(key=self._orderkey)
 
     def _add_node(self, node):
-        """Adds a :class:node to the :class:Layer.
+        """Adds a :class:`node` to the :class:`Layer`.
 
-        Assumes node has no incoming or outgoing :class:Edge objects.
+        Assumes node has no incoming or outgoing :class:`Edge` objects.
 
         """
         self._all.append(node)
@@ -752,27 +853,27 @@ class Layer:
         self._heads.sort(key=self._orderkey)
 
     def _remove_node(self, node):
-        """Removes a :class:node from the :class:Layer.
+        """Removes a :class:`node` from the :class:`Layer`.
 
-        Assumes node has no incoming or outgoing :class:Edge objects.
+        Assumes node has no incoming or outgoing :class:`Edge` objects.
 
         """
         self._all.remove(node)
         self._heads.remove(node)
 
     def _change_edge_tag(self, edge, old_tag):
-        """Updates the :class:Layer objects with the change.
+        """Updates the :class:`Layer` objects with the change.
 
-        :param edge: the updated :class:Edge object
+        :param edge: the updated :class:`Edge` object
             old_tag: the Edge's tag before the change
 
         """
         pass  # meant to be overriden by subclasses
 
     def _change_node_tag(self, node, old_tag):
-        """Updates the :class:Layer objects with the change.
+        """Updates the :class:`Layer` objects with the change.
 
-        :param node: the updated :class:Node object
+        :param node: the updated :class:`Node` object
             old_tag: the Node's tag before the change
 
         """
@@ -783,8 +884,8 @@ class Passage:
     """An annotated text with UCCA annotation graph.
 
     A Passage is an object representing a text annotated with UCCA annotation.
-    UCCA annotation is a directed acyclic graph of :class:Node and :class:Edge
-    objects grouped into :class:Layer objects.
+    UCCA annotation is a directed acyclic graph of :class:`Node` and :class:`Edge`
+    objects grouped into :class:`Layer` objects.
 
     Attributes:
         ID: ID of the Passage
@@ -798,9 +899,9 @@ class Passage:
     """
 
     def __init__(self, ID, attrib=None):
-        """Creates a new :class:Passage object.
+        """Creates a new :class:`Passage` object.
 
-        :param see :class:Passage documentation.
+        :param see :class:`Passage` documentation.
 
         """
         self._ID = ID
@@ -808,6 +909,8 @@ class Passage:
         self.extra = {}
         self._layers = {}
         self._nodes = {}
+        self._categories = {}
+        self._refined_categories = []
         self.frozen = False
 
     @property
@@ -830,15 +933,26 @@ class Passage:
     def nodes(self):
         return self._nodes.copy()
 
+    @property
+    def categories(self):
+        return self._categories.copy()
+
+    @property
+    def refined_categories(self):
+        return self._refined_categories
+
     def layer(self, ID):
-        """Returns the :class:Layer object whose ID is given.
+        """Returns the :class:`Layer` object whose ID is given.
 
         :param ID: ID of the Layer requested.
 
         :raise KeyError: if no Layer with this ID is present
 
         """
-        return self._layers[ID]
+        try:
+            return self._layers[ID]
+        except KeyError as e:
+            raise KeyError("Layer '%s' not found in passage '%s'" % (ID, self.ID)) from e
 
     def equals(self, other, *, ordered=False, ignore_node=None, ignore_edge=None):
         """Returns whether two passages are equivalent.
@@ -851,7 +965,7 @@ class Passage:
         :param ignore_node: function that returns whether to ignore a given node
         :param ignore_edge: function that returns whether to ignore a given edge
 
-        :return True iff self is Passage-equivalent to other.
+        :return: True iff self is Passage-equivalent to other.
 
         """
         if not self._attrib.equals(other._attrib):
@@ -876,7 +990,7 @@ class Passage:
         :param ignore_node: function that returns whether to ignore a given node
         :param ignore_edge: function that returns whether to ignore a given edge
 
-        :return List of nodes present in this passage but missing in the other.
+        :return: List of nodes present in this passage but missing in the other.
 
         """
         nodes, other_nodes = [[node for node in passage.nodes.values()
@@ -889,7 +1003,7 @@ class Passage:
                                   for n2 in other_nodes)],
                       key=id_orderkey)
 
-    def copy(self, layers):
+    def copy(self, layers=None):
         """Copies the Passage and specified layers to a new object.
 
         The main "building block" of copying is the Layer, so copying is
@@ -897,20 +1011,23 @@ class Passage:
         and creating the equivalent layers (each layer for itself).
 
         :param layers: sequence of layer IDs to copy to the new object.
+            If None, all layers will be copied.
 
-        :return A new Passage object.
+        :return: A new Passage object.
 
-        :raise KeyError if a given layer ID doesn't exist.
+        :raise KeyError: if a given layer ID doesn't exist.
             UnimplementedMethodError if copying for a layer is unimplemented.
 
         """
         other = Passage(ID=self.ID, attrib=self.attrib.copy())
         other.extra = self.extra.copy()
+        if layers is None:
+            layers = sorted(self._layers)
         for lid in layers:
             try:
                 self.layer(lid).copy(other)
-            except AttributeError:
-                raise UnimplementedMethodError()
+            except AttributeError as e:
+                raise UnimplementedMethodError() from e
         other.frozen = self.frozen
         return other
 
@@ -918,51 +1035,62 @@ class Passage:
         """Returns a Node whose ID is given.
 
         :param ID: ID string
-
-        :return The node.Node object whose ID matches
-
-        :raise KeyError if no Node with this ID is found
-
+        :return: The node.Node object whose ID matches
+        :raise KeyError: if no Node with this ID is found
         """
-        return self._nodes[ID]
+        try:
+            return self._nodes[ID]
+        except KeyError as e:
+            layer_id, _ = ID.split(Node.ID_SEPARATOR)
+            layer = self.layer(layer_id)
+            raise KeyError("Node '%s' not found in passage '%s', existing IDs in layer '%s': %s" % (
+                ID, self.ID, layer_id, ", ".join(n.ID for n in layer.all))) from e
 
     @ModifyPassage
     def _add_layer(self, layer):
-        """Adds a :class:Layer object to the :class:Passage.
+        """Adds a :class:`Layer` object to the :class:`Passage`.
 
         :param layer: the Layer object to add
 
         :raise DuplicateIdError: if layer.ID is identical to a Layer already
                 present in the Passage.
-            FrozenPassageError: if the :class:Passage object we are part of
+            FrozenPassageError: if the :class:`Passage` object we are part of
                 is frozen and can't be modified.
 
         """
         if layer.ID in self._layers:
-            raise DuplicateIdError()
+            raise DuplicateIdError(layer.ID)
         self._layers[layer.ID] = layer
 
+    @ModifyPassage
+    def _update_categories(self, category):
+        self._categories[category.tag] = {"layer": category.layer, "slot": category.slot, "parent": category.parent}
+
+    @ModifyPassage
+    def _update_refined_categories(self, refined_category):
+        self._refined_categories.append(refined_category)
+
     @ModifyPassage
     def _add_node(self, node):
-        """Adds a :class:Node object to the :class:Passage.
+        """Adds a :class:`Node` object to the :class:`Passage`.
 
         :param node: the Node object to add
 
         :raise DuplicateIdError: if node.ID is identical to a Node already
                 present in the Passage.
-            FrozenPassageError: if the :class:Passage object we are part of
+            FrozenPassageError: if the :class:`Passage` object we are part of
                 is frozen and can't be modified.
 
         """
         if node.ID in self._nodes:
-            raise DuplicateIdError()
+            raise DuplicateIdError(node.ID)
         self._nodes[node.ID] = node
 
     def _remove_node(self, node):
-        """Removes a :class:Node object from the :class:Passage.
+        """Removes a :class:`Node` object from the :class:`Passage`.
 
         :param node: the Node object to remove, must be unlinked with any other
-                Node objects and removed from its :class:Layer.
+                Node objects and removed from its :class:`Layer`.
 
         :raise KeyError: if no Node with this ID is present
 
@@ -971,9 +1099,9 @@ class Passage:
 
     @ModifyPassage
     def _add_edge(self, edge):
-        """Adds a :class:Edge object to :class:Passage.
+        """Adds a :class:`Edge` object to :class:`Passage`.
 
-        Handles altering the Passage and :class:Layer objects accordingly.
+        Handles altering the Passage and :class:`Layer` objects accordingly.
 
         :param edge: the Edge object to add
 
@@ -982,9 +1110,9 @@ class Passage:
         edge.parent.layer._add_edge(edge)
 
     def _remove_edge(self, edge):
-        """Removes a :class:Edge object from :class:Passage.
+        """Removes a :class:`Edge` object from :class:`Passage`.
 
-        Handles altering the Passage and :class:Layer objects accordingly.
+        Handles altering the Passage and :class:`Layer` objects accordingly.
 
         :param edge: the Edge object to remove
 
@@ -993,9 +1121,9 @@ class Passage:
         edge.parent.layer._remove_edge(edge)
 
     def _change_edge_tag(self, edge, old_tag):
-        """Updates the :class:Passage and :class:Layer objects with the change.
+        """Updates the :class:`Passage` and :class:`Layer` objects with the change.
 
-        :param edge: the updated :class:Edge object
+        :param edge: the updated :class:`Edge` object
             old_tag: the Edge's tag before the change
 
         """
@@ -1003,9 +1131,9 @@ class Passage:
         edge.parent.layer._change_edge_tag(edge, old_tag)
 
     def _change_node_tag(self, node, old_tag):
-        """Updates the :class:Passage and :class:Layer objects with the change.
+        """Updates the :class:`Passage` and :class:`Layer` objects with the change.
 
-        :param node: the updated :class:Node object
+        :param node: the updated :class:`Node` object
             old_tag: the Node's tag before the change
 
         """
