@@ -5,7 +5,8 @@ from ucca.layer1 import EdgeTags as ETags, NodeTags as L1Tags
 NO_MULTIPLE_INCOMING_CATEGORIES = {ETags.Function, ETags.ParallelScene, ETags.Linker, ETags.LinkRelation,
                                    ETags.Connector, ETags.Punctuation, ETags.Terminal}
 TOP_CATEGORIES = {ETags.ParallelScene, ETags.Linker, ETags.Function, ETags.Ground, ETags.Punctuation,
-                  ETags.LinkRelation, ETags.LinkArgument, ETags.Connector}
+                  ETags.Relator, ETags.LinkRelation, ETags.LinkArgument, ETags.Connector}
+KEEP_OUTSIDE_CMR = {ETags.Relator}
 COORDINATED_MAIN_REL = "Coordinated_Main_Rel."
 
 
@@ -177,6 +178,14 @@ def detach_punct(l1):
     for node in l1.all:
         if node.tag == L1Tags.Punctuation:
             destroy(node)
+        else:
+            to_remove = []
+            for edge in node:
+                if edge.child.tag == L0Tags.Punct:
+                    to_remove.append(edge)
+            if len(node) < len(to_remove):  # Detach only if we are not making the node childless
+                for edge in to_remove:
+                    destroy(edge)
 
 
 def reattach_terminals(l0, l1):
@@ -239,7 +248,7 @@ def flatten_functions(node):
 def flatten_participants(node):
     """
     Whenever there is an A as an only child, remove it.
-    If there is an implicit A in a scene without a main relation, remove it.
+    If there is an implicit A in a unit without a main relation, remove it.
     """
     if node.tag == L1Tags.Foundational:
         participants = node.participants
@@ -254,6 +263,19 @@ def flatten_participants(node):
     return node
 
 
+def flatten_scenes(node):
+    """
+    Whenever there is an H with H inside, remove the top one
+    """
+    if node.tag == L1Tags.Foundational:
+        for ps in node.parallel_scenes:
+            if ps and all(TOP_CATEGORIES.intersection(edge.tags) for edge in ps.outgoing):
+                for edge in ps.outgoing:
+                    copy_edge(edge, parent=node)
+                    destroy(edge)
+    return node
+
+
 def split_coordinated_main_rel(node, l1):
     for edge in node:
         attrib = edge.attrib.copy()
@@ -272,14 +294,18 @@ def split_coordinated_main_rel(node, l1):
             else:
                 top = node
             outgoing = list(node.outgoing)
+            external = [e for e in outgoing if KEEP_OUTSIDE_CMR.intersection(e.tags)]
+            internal = [e for e in outgoing if not KEEP_OUTSIDE_CMR.intersection(e.tags) and
+                        e.ID != edge.ID]  # Not the CMR edge itself
+            for scene_edge in external:  # A category that should be kept outside of both scenes
+                copy_edge(scene_edge, parent=top)
             scenes = []
             for center in centers:
                 new_scene = l1.add_fnode(top, ETags.ParallelScene)
                 copy_edge(edge, parent=new_scene, child=center, attrib=attrib)
-                for scene_edge in outgoing:
-                    if scene_edge.ID != edge.ID and not (
-                            scenes and NO_MULTIPLE_INCOMING_CATEGORIES.intersection(scene_edge.tags)):
-                        # Not the CMR edge itself, and not a category that does not allow multiple parents
+                for scene_edge in internal:
+                    if not (scenes and NO_MULTIPLE_INCOMING_CATEGORIES.intersection(scene_edge.tags)):
+                        # Attach inside the 1st scene and potentially also as remote in 2nd scene
                         copy_edge(scene_edge, parent=new_scene, attrib={"remote": True} if scenes else None)
                 scenes.append(new_scene)
             for main_rel_edge in main_rel_non_centers:
@@ -313,7 +339,10 @@ def normalize_node(node, l1, extra):
         node = flatten_functions(node)
         if node is None:
             return
-        flatten_participants(node)
+        node = flatten_participants(node)
+        if node is None:
+            return
+        flatten_scenes(node)
 
 
 def normalize(passage, extra=False):
