@@ -81,6 +81,36 @@ def unary_function():
     return p
 
 
+def unary_participant():
+    """
+    Whenever there is an A as an only child, remove it.
+    """
+    p, l1, terms = create_passage()
+    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
+    a = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
+    ps2 = l1.add_fnode(a, layer1.EdgeTags.ParallelScene)
+    p1 = l1.add_fnode(ps2, layer1.EdgeTags.Process)
+    linker = l1.add_fnode(a, layer1.EdgeTags.Linker)
+    ps3 = l1.add_fnode(a, layer1.EdgeTags.ParallelScene)
+    p2 = l1.add_fnode(ps3, layer1.EdgeTags.Process)
+    attach_terminals(terms, p1, linker, p2)
+    return p
+
+
+def simple_linkage():
+    """
+    Whenever there is an A as an only child, remove it.
+    """
+    p, l1, terms = create_passage()
+    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
+    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
+    linker = l1.add_fnode(None, layer1.EdgeTags.Linker)
+    ps2 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
+    p2 = l1.add_fnode(ps2, layer1.EdgeTags.Process)
+    attach_terminals(terms, p1, linker, p2)
+    return p
+
+
 def simple_function():
     p, l1, terms = create_passage()
     ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
@@ -286,15 +316,20 @@ def cmr_no_attrib():
     return p
 
 
+def insert_cmr(l1, scene):
+    p = l1.add_fnode(scene, layer1.EdgeTags.Process)
+    p.incoming[0].attrib[COORDINATED_MAIN_REL] = True
+    c1 = l1.add_fnode(p, layer1.EdgeTags.Center)
+    n = l1.add_fnode(p, layer1.EdgeTags.Connector)
+    c2 = l1.add_fnode(p, layer1.EdgeTags.Center)
+    return p, c1, n, c2
+
+
 def cmr():
     p, l1, terms = create_passage(4)
     ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
     a = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
-    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
-    p1.incoming[0].attrib[COORDINATED_MAIN_REL] = True
-    c1 = l1.add_fnode(p1, layer1.EdgeTags.Center)
-    n = l1.add_fnode(p1, layer1.EdgeTags.Connector)
-    c2 = l1.add_fnode(p1, layer1.EdgeTags.Center)
+    p1, c1, n, c2 = insert_cmr(l1, ps1)
     attach_terminals(terms, a, c1, n, c2)
     return p
 
@@ -318,11 +353,7 @@ def remote_cmr():
     linker = l1.add_fnode(None, layer1.EdgeTags.Linker)
     ps2 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
     a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
-    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
-    p1.incoming[0].attrib[COORDINATED_MAIN_REL] = True
-    c1 = l1.add_fnode(p1, layer1.EdgeTags.Center)
-    n = l1.add_fnode(p1, layer1.EdgeTags.Connector)
-    c2 = l1.add_fnode(p1, layer1.EdgeTags.Center)
+    p1, c1, n, c2 = insert_cmr(l1, ps1)
     a2 = l1.add_fnode(ps2, layer1.EdgeTags.Participant)
     l1.add_remote(ps2, layer1.EdgeTags.Process, p1)
     attach_terminals(terms, a1, c1, n, c2, linker, a2)
@@ -349,6 +380,47 @@ def expanded_remote_cmr():
     return p
 
 
+def cmr_with_relator_outside():
+    """
+    [suitable_S [for_R the_F [breeding_C and_N growing_C]_P|CMR [of_R giants_C]_A]_A]_H
+    """
+    p, l1, terms = create_passage(8)
+    ps = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
+    s = l1.add_fnode(ps, layer1.EdgeTags.State)
+    a_scene = l1.add_fnode(ps, layer1.EdgeTags.Participant)
+    r1 = l1.add_fnode(a_scene, layer1.EdgeTags.Relator)
+    f = l1.add_fnode(a_scene, layer1.EdgeTags.Function)
+    p1, c1, n, c2 = insert_cmr(l1, a_scene)
+    a2 = l1.add_fnode(a_scene, layer1.EdgeTags.Participant)
+    r2 = l1.add_fnode(a2, layer1.EdgeTags.Relator)
+    c3 = l1.add_fnode(a2, layer1.EdgeTags.Center)
+    attach_terminals(terms, s, r1, f, c1, n, c2, r2, c3)
+    return p
+
+
+def expanded_cmr_with_relator_outside():
+    """
+    [suitable_S [for_R [[the_F breeding_C]_P [of_R giants_C]_A]_H and_L [[growing_P] [*of giants]_A]_H]_H
+    """
+    p, l1, terms = create_passage(8)
+    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
+    s = l1.add_fnode(ps1, layer1.EdgeTags.State)
+    a_superscene = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
+    r1 = l1.add_fnode(a_superscene, layer1.EdgeTags.Relator)
+    ps2 = l1.add_fnode(a_superscene, layer1.EdgeTags.ParallelScene)
+    linker = l1.add_fnode(a_superscene, layer1.EdgeTags.Linker)
+    ps3 = l1.add_fnode(a_superscene, layer1.EdgeTags.ParallelScene)
+    p1 = l1.add_fnode(ps2, layer1.EdgeTags.Process)
+    f = l1.add_fnode(ps2, layer1.EdgeTags.Function)
+    a = l1.add_fnode(ps2, layer1.EdgeTags.Participant)
+    r2 = l1.add_fnode(a, layer1.EdgeTags.Relator)
+    c2 = l1.add_fnode(a, layer1.EdgeTags.Center)
+    p2 = l1.add_fnode(ps3, layer1.EdgeTags.Process)
+    l1.add_remote(ps3, layer1.EdgeTags.Participant, a)
+    attach_terminals(terms, s, r1, f, p1, linker, p2, r2, c2)
+    return p
+
+
 def normalize_and_compare(unnormalized, normalized, extra=False):
     p1 = unnormalized()
     p2 = normalized()
@@ -380,6 +452,8 @@ def normalize_and_compare(unnormalized, normalized, extra=False):
         (cmr_no_attrib, cmr_no_attrib),
         (cmr, expanded_cmr),
         (remote_cmr, expanded_remote_cmr),
+        (cmr_with_relator_outside, expanded_cmr_with_relator_outside),
+        (unary_participant, simple_linkage),
 ))
 def test_normalize(unnormalized, normalized):
     normalize_and_compare(unnormalized, normalized)
