@@ -60,7 +60,7 @@ class TaskUploader(ServerAccessor):
                 if log:
                     log_h.close()
 
-    def upload_task(self, passage, log=None, submit=True, ids=None):
+    def upload_task(self, passage, log=None, submit=True, ids=None, upload=True):
         if ids:
             passage_id, tok_id, ann_id = ids[passage.ID]
             passage_out = self.get_passage(passage_id)
@@ -68,19 +68,19 @@ class TaskUploader(ServerAccessor):
             ann_user_task_in = self.get_user_task(ann_id)
         else:
             passage_out = self.create_passage(text=to_text(passage, sentences=False)[0], type="PUBLIC",
-                                              source=self.source, external_id=passage.ID)
+                                              source=self.source, external_id=passage.ID) if upload else passage
             task_in = dict(type="TOKENIZATION", status="ONGOING", project=self.project, user=self.user,
                            passage=passage_out, manager_comment=passage.ID, user_comment=passage.ID, parent=None,
                            is_demo=False, is_active=True)
-            tok_task_out = self.create_task(**task_in)
+            tok_task_out = self.create_task(**task_in) if upload else task_in
             tok_user_task_in = dict(tok_task_out)
             tok_user_task_in.update(to_json(passage, return_dict=True, tok_task=True))
-            tok_user_task_out = self.submit_task(**tok_user_task_in)
+            tok_user_task_out = self.submit_task(**tok_user_task_in) if upload else tok_user_task_in
             task_in.update(parent=tok_task_out, type="ANNOTATION")
-            ann_user_task_in = self.create_task(**task_in)
+            ann_user_task_in = self.create_task(**task_in) if upload else task_in
         ann_user_task_in.update(
             to_json(passage, return_dict=True, tok_task=tok_user_task_out, all_categories=self.layer["categories"]))
-        ann_user_task_out = self.submit_task(**ann_user_task_in, submit=submit)
+        ann_user_task_out = self.submit_task(**ann_user_task_in, submit=submit) if upload else ann_user_task_in
         if log:
             print(passage.ID, passage_out["id"], tok_task_out["id"], ann_user_task_out["id"],
                   file=log, sep="\t", flush=True)
@@ -92,6 +92,7 @@ class TaskUploader(ServerAccessor):
         argparser.add_argument("-l", "--log", help="filename to write log of uploaded passages to")
         argparser.add_argument("--no-submit", action="store_false", dest="submit", help="do not submit annotation task")
         argparser.add_argument("--existing-ids", help="use existing task IDs from file (output of --log); no creation")
+        argparser.add_argument("-n", "--no-upload", action="store_false", dest="upload", help="do not upload anything")
         ServerAccessor.add_project_id_argument(argparser)
         ServerAccessor.add_source_id_argument(argparser)
         ServerAccessor.add_user_id_argument(argparser)
